/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect;
public class EnumBiMapTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testModule";
}
public void testCreate() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testCreate();
}

public void testCreateFromMap() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testCreateFromMap();
}

public void testEntrySet() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testEntrySet();
}

public void testEnumBiMapConstructor() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testEnumBiMapConstructor();
}

public void testEquals() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testEquals();
}

public void testIterationOrder() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testIterationOrder();
}

public void testKeySetIteratorRemove() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testKeySetIteratorRemove();
}

public void testKeyType() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testKeyType();
}

public void testValueType() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testValueType();
}

public void testValuesIteratorRemove() throws Exception {
  com.google.common.collect.EnumBiMapTest testCase = new com.google.common.collect.EnumBiMapTest();
  testCase.testValuesIteratorRemove();
}
}
