/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.collect.testing;
public class HelpersTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.collect.testing.testModule";
}
public void testAssertContains() throws Exception {
  com.google.common.collect.testing.HelpersTest testCase = new com.google.common.collect.testing.HelpersTest();
  testCase.testAssertContains();
}

public void testAssertContainsAllOf() throws Exception {
  com.google.common.collect.testing.HelpersTest testCase = new com.google.common.collect.testing.HelpersTest();
  testCase.testAssertContainsAllOf();
}

public void testAssertContentsInOrder() throws Exception {
  com.google.common.collect.testing.HelpersTest testCase = new com.google.common.collect.testing.HelpersTest();
  testCase.testAssertContentsInOrder();
}

public void testAssertEqualInOrder() throws Exception {
  com.google.common.collect.testing.HelpersTest testCase = new com.google.common.collect.testing.HelpersTest();
  testCase.testAssertEqualInOrder();
}

public void testIsEmpty_iterable() throws Exception {
  com.google.common.collect.testing.HelpersTest testCase = new com.google.common.collect.testing.HelpersTest();
  testCase.testIsEmpty_iterable();
}

public void testIsEmpty_map() throws Exception {
  com.google.common.collect.testing.HelpersTest testCase = new com.google.common.collect.testing.HelpersTest();
  testCase.testIsEmpty_map();
}

public void testNullsBeforeB() throws Exception {
  com.google.common.collect.testing.HelpersTest testCase = new com.google.common.collect.testing.HelpersTest();
  testCase.testNullsBeforeB();
}
}
