/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class JoinerTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testEntries() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.testEntries();
}

public void testMap() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.testMap();
}

public void testNoSpecialNullBehavior() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.testNoSpecialNullBehavior();
}

public void testOnCharOverride() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.testOnCharOverride();
}

public void testSkipNulls() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.testSkipNulls();
}

public void testUseForNull() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.testUseForNull();
}

public void test_skipNulls_onMap() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.test_skipNulls_onMap();
}

public void test_skipNulls_useForNull() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.test_skipNulls_useForNull();
}

public void test_useForNull_skipNulls() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.test_useForNull_skipNulls();
}

public void test_useForNull_twice() throws Exception {
  com.google.common.base.JoinerTest testCase = new com.google.common.base.JoinerTest();
  testCase.test_useForNull_twice();
}
}
