/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.primitives;
public class BooleansTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.primitives.testModule";
}
public void testAsListContains() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListContains();
}

public void testAsListEquals() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListEquals();
}

public void testAsListHashcode() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListHashcode();
}

public void testAsListIndexOf() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListIndexOf();
}

public void testAsListIsEmpty() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListIsEmpty();
}

public void testAsListLastIndexOf() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListLastIndexOf();
}

public void testAsListSet() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListSet();
}

public void testAsListSize() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListSize();
}

public void testAsListToString() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testAsListToString();
}

public void testCompare() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testCompare();
}

public void testConcat() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testConcat();
}

public void testContains() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testContains();
}

public void testCountTrue() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testCountTrue();
}

public void testEnsureCapacity() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testEnsureCapacity();
}

public void testEnsureCapacity_fail() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testEnsureCapacity_fail();
}

public void testHashCode() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testHashCode();
}

public void testIndexOf() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testIndexOf();
}

public void testIndexOf_arrays() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testIndexOf_arrays();
}

public void testJoin() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testJoin();
}

public void testLastIndexOf() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testLastIndexOf();
}

public void testLexicographicalComparator() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testLexicographicalComparator();
}

public void testToArray() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testToArray();
}

public void testToArray_threadSafe() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testToArray_threadSafe();
}

public void testToArray_withNull() throws Exception {
  com.google.common.primitives.BooleansTest testCase = new com.google.common.primitives.BooleansTest();
  testCase.testToArray_withNull();
}
}
