/*
 * Copyright (C) 2008 The Guava Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.common.base;
public class VerifyTest_gwt extends com.google.gwt.junit.client.GWTTestCase {
@Override public String getModuleName() {
  return "com.google.common.base.testModule";
}
public void testVerifyNotNull_complexMessage_success() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerifyNotNull_complexMessage_success();
}

public void testVerifyNotNull_simpleMessage_failure() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerifyNotNull_simpleMessage_failure();
}

public void testVerifyNotNull_simple_failure() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerifyNotNull_simple_failure();
}

public void testVerifyNotNull_simple_success() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerifyNotNull_simple_success();
}

public void testVerify_complexMessage_failure() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerify_complexMessage_failure();
}

public void testVerify_complexMessage_success() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerify_complexMessage_success();
}

public void testVerify_simpleMessage_failure() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerify_simpleMessage_failure();
}

public void testVerify_simpleMessage_success() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerify_simpleMessage_success();
}

public void testVerify_simple_failure() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerify_simple_failure();
}

public void testVerify_simple_success() throws Exception {
  com.google.common.base.VerifyTest testCase = new com.google.common.base.VerifyTest();
  testCase.testVerify_simple_success();
}
}
